diff -uNr jpgal11/jpeg.c jpgal11b/jpeg.c
--- jpgal11/jpeg.c	Fri Jun  2 11:55:16 2000
+++ jpgal11b/jpeg.c	Fri Nov 16 21:27:38 2001
@@ -24,6 +24,7 @@
 #include <allegro/aintern.h>
 #include <stdio.h>
 #include <math.h>
+#include "jpgalleg.h"
 
 
 #define FIX_0_298631336  ((long)  2446)
@@ -65,6 +66,10 @@
 #define JFIF_OK         0x3F
 
 
+static jpg_progress_monitor progress_monitor = NULL;
+static void * progress_monitor_data = NULL;
+static void report_progress(int completed, int total);
+
 typedef struct HUFFMAN_NODE
 {
    long index;
@@ -94,6 +99,29 @@
 
 
 
+/* report_progress:
+ *  Reports progress to the progress monitor, if it exists;
+ */
+static void report_progress(int completed, int total)
+{
+    if (progress_monitor != NULL)
+    {
+        (*progress_monitor)(progress_monitor_data, completed, total);
+    }
+}
+
+/* jpg_register_progress_monitor:
+ *  Registers a progress monitor.  Passing null, unregisters the current
+ *  monitor.
+ */
+
+void jpg_register_progress_monitor(void * data, jpg_progress_monitor monitor)
+{
+    progress_monitor        = monitor;
+    progress_monitor_data   = data;
+}
+
+
 /* next_bit:
  *  Returns next bit from decoding data stream.
  */
@@ -781,6 +809,7 @@
                         dec.data += 2;
                         dec.bits = 0;
                      }
+                     report_progress(MIN(y, dec.height), dec.height);
                   }
 
                } while (y < dec.height);
@@ -815,6 +844,7 @@
                         dec.data += 2;
                         dec.bits = 0;
                      }
+                     report_progress(MIN(y, dec.height), dec.height);
                   }
 
                } while (y < dec.height);
@@ -842,11 +872,13 @@
                         dec.data += 2;
                         dec.bits = 0;
                      }
+                     report_progress(MIN(y, dec.height), dec.height);
                   }
 
                } while (y < dec.height);
                break;
          }
+         break;
 
       case 1:
          /*
@@ -868,6 +900,7 @@
                   dec.data += 2;
                   dec.bits = 0;
                }
+               report_progress(MIN(y, dec.height), dec.height);
             }
 
          } while (y < dec.height);
@@ -886,7 +919,7 @@
 /* load_jpg:
  *  Loads a jpg image file (duh!).
  */
-BITMAP *load_jpg(char *filename, RGB *pal)
+BITMAP *load_jpg(const char *filename, RGB *pal)
 {
    PACKFILE *f;
    BITMAP *bmp;
diff -uNr jpgal11/jpgalleg.h jpgal11b/jpgalleg.h
--- jpgal11/jpgalleg.h	Fri Jun  2 15:04:44 2000
+++ jpgal11b/jpgalleg.h	Fri Nov 16 21:41:44 2001
@@ -9,6 +9,10 @@
 #ifndef JPGALLEG_H
 #define JPGALLEG_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include <allegro.h>
 
 /* load_memory_jpg:
@@ -19,6 +23,17 @@
 /* load_jpg:
  *  Decodes a JPG image from a standard JPG file.
  */
-BITMAP *load_jpg(char *filename, RGB *pal);
+BITMAP *load_jpg(const char *filename, RGB *pal);
+
+typedef void (*jpg_progress_monitor)(void * data, int completed, int total);
+
+/* jpg_register_progress_monitor:
+ *  Registers a progress monitor callback.
+ */
+void jpg_register_progress_monitor(void * data, jpg_progress_monitor monitor);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif
diff -uNr jpgal11/makefile.dj jpgal11b/makefile.dj
--- jpgal11/makefile.dj	Thu Jan  1 00:00:00 1970
+++ jpgal11b/makefile.dj	Fri Nov 16 21:43:10 2001
@@ -0,0 +1,28 @@
+all: jpegdemo ex1 ex2 libjpgal.a
+
+jpeg.o: jpeg.c jpgalleg.h
+	gcc -O3 -c jpeg.c -o jpeg.o
+
+libjpgal.a: jpeg.o
+	ar rs libjpgal.a jpeg.o
+
+jpegdemo: jpegdemo.o jpeg.o
+	gcc -o jpegdemo.exe jpegdemo.o jpeg.o -lalleg
+
+jpegdemo.o: jpegdemo.c
+	gcc -O3 -c jpegdemo.c -o jpegdemo.o
+
+ex1: ex1.o
+	gcc -o ex1.exe ex1.o jpeg.o -lalleg
+
+ex1.o: ex1.c
+	gcc -O3 -c ex1.c -o ex1.o
+
+ex2: ex2.o
+	gcc -o ex2.exe ex2.o jpeg.o -lalleg
+
+ex2.o: ex2.c
+	gcc -O3 -c ex2.c -o ex2.o
+
+clean:
+	$(RM) *.o *.a *.exe *~
